@@ -126,6 +126,14 @@ just satisfies the Promises/A+ convention.
 Any promise created while C<$Promise::ES6::DETECT_MEMORY_LEAKS> is truthy
 will throw a warning if it survives until global destruction.
 
+=head1 SUBCLASSING
+
+You can re-bless a L<Promise::XS::Promise> instance into a different class,
+and C<then()>, C<catch()>, and C<finally()> will assign their newly-created
+promise into that other class. (It follows that the other class must subclass
+L<Promise::XS::Promise>.) This can be useful, e.g., for implementing
+mid-flight controls like cancellation.
+
 =head1 TODO
 
 =over
@@ -185,20 +193,27 @@ sub all {
     my $then_what= deferred();
     my $pending= 1;
     my $i= 0;
+
+    my $reject_now = sub {
+        if (!$failed++) {
+            $pending= 0;
+            $then_what->reject(@_);
+        }
+    };
+
     for my $p (@_) {
-        my $i= $i++;
-        $p->then(sub {
-            $values[$i]= \@_;
-            if ((--$remaining) == 0) {
-                $pending= 0;
-                $then_what->resolve(@values);
-            }
-        }, sub {
-            if (!$failed++) {
-                $pending= 0;
-                $then_what->reject(@_);
-            }
-        });
+        my $i = $i++;
+
+        $p->then(
+            sub {
+                $values[$i]= \@_;
+                if ((--$remaining) == 0) {
+                    $pending= 0;
+                    $then_what->resolve(@values);
+                }
+            },
+            $reject_now,
+        );
     }
     if (!$remaining && $pending) {
         $then_what->resolve(@values);
@@ -212,35 +227,34 @@ sub all {
 # Lifted from Promise::ES6
 sub race {
 
-    # Perl 5.16 and earlier leak memory when the callbacks are handled
-    # inside the closure here.
     my $deferred = deferred();
 
     my $is_done;
 
+    my $on_resolve_cr = sub {
+        return if $is_done;
+        $is_done = 1;
+
+        $deferred->resolve(@_);
+
+        # Proactively eliminate references:
+        undef $deferred;
+    };
+
+    my $on_reject_cr = sub {
+        return if $is_done;
+        $is_done = 1;
+
+        $deferred->reject(@_);
+
+        # Proactively eliminate references:
+        undef $deferred;
+    };
+
     for my $given_promise (@_) {
         last if $is_done;
 
-        $given_promise->then(
-            sub {
-                return if $is_done;
-                $is_done = 1;
-
-                $deferred->resolve(@_);
-
-                # Proactively eliminate references:
-                undef $deferred;
-            },
-            sub {
-                return if $is_done;
-                $is_done = 1;
-
-                $deferred->reject(@_);
-
-                # Proactively eliminate references:
-                undef $deferred;
-            }
-        );
+        $given_promise->then($on_resolve_cr, $on_reject_cr);
     }
 
     return $deferred->promise();
